/*
 Copyright (c) 2007, Distributed Computing Group (DCG)
                    ETH Zurich
                    Switzerland
                    dcg.ethz.ch

 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the
   distribution.

 - Neither the name 'Sinalgo' nor the names of its contributors may be
   used to endorse or promote products derived from this software
   without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package projects.defaultProject.models.connectivityModels;

import sinalgo.configuration.Configuration;
import sinalgo.configuration.CorruptConfigurationEntryException;
import sinalgo.models.ConnectivityModelHelper;
import sinalgo.nodes.Node;
import sinalgo.nodes.Position;
import sinalgo.runtime.Global;
import sinalgo.runtime.Main;
import sinalgo.tools.Tools;

/**
 * Class to implement the SINR Connectivity Model. This means, that a nodes is connected to another node
 * if and only if the SNR constraint (without interference) is fulfilled.
 */
public class SINRg extends ConnectivityModelHelper  implements RangeConnectivityModel{
		
	/* (non-Javadoc)
	 * @see connectivityModels.ConnectivityModel#isConnected(nodes.Node, nodes.Node)
	 */
	protected boolean isConnected(Node from, Node to){
		
		double range = Math.pow(from.getRadioIntensity()/(betatimescnoise), oneoveralpha);
		//Tools.appendToOutput(" " + range + "?" + p1.distanceTo(p2));
		//Global.log.logln("range " + range + " distance " + p1.distanceTo(p2));
		
		return (from.getPosition().distanceTo(to.getPosition()) < range);
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	// Code to initialize the static variables of this class 
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -   
	
	private static boolean initialized = false; // indicates whether the static fields of this class have already been initialized 
	private static double alpha;
	private static double oneoveralpha; // SINR configuration values
	private static double betatimescnoise; // SINR configuration values
	
	/* 
	 * compute the range of the node based on its radio intensity
	 */
	public double getRange(Node n) {
		return getRange(n.getRadioIntensity());
	}
	
	public double getRange(double p) {
		return Math.pow(p/(betatimescnoise), oneoveralpha);
	}
	
	/* 
	 * compute the power required for a node to achieve a given range
	 */
	public double getPower(double range) {
		return Math.pow(range, alpha)*betatimescnoise;
	}
	
	/**
	 * The default constructor for this class.  
	 * 
	 * The first time this constructor is called, it initializes the static parameters of this class. 
	 * @throws CorruptConfigurationEntryException If one of the initialization steps fails.
	 */
	public SINRg() throws CorruptConfigurationEntryException {
		if(! initialized) {
			//zero initialization is not a problem, as we throw fatal errors if values are not set
			alpha = 0;
			double beta = 0;
			double noise = 0;
			double c = 0;
			try{
				alpha = Configuration.getDoubleParameter("SINR/alpha");
				beta = Configuration.getDoubleParameter("SINR/beta");
				noise = Configuration.getDoubleParameter("SINR/noise");
				c = Configuration.getDoubleParameter("SINR/c");
			} catch(CorruptConfigurationEntryException e) {
				Tools.fatalError("The configuration entry SINR/(alpha, beta, noise or c)is not a valid double:\n\n" + e.getMessage());
			}
			
			oneoveralpha = 1/alpha;
			betatimescnoise = beta*c*noise;
			//Tools.appendToOutput(" 1/a=" + oneoveralpha + " bcn=" + betatimescnoise);
			
			initialized = true;
		}
	}
}

